import {
    select,
    csv,
    scaleLinear,
    extent,
    max,
    axisLeft,
    axisBottom
} from 'd3';
const svg =d3.select('svg');
var lineg = svg.append('g');
var dotg = svg.append('g');
var centerg = svg.append('g');
var flag = false;

var groups=[];//用来保存每组的信息
var dots=[];//来存放所有点的位置信息，和分组信息

let N=d3.select("#nValue").property("value");
let K=d3.select("#kValue").property("value");



export const scatterPlot = (selection, props) => {
    const {
        xValue,
        xAxisLabel,
        yValue,
        circleRadius,
        yAxisLabel,
        margin,
        width,
        height,

    } = props;

    const innerWidth = width - margin.left - margin.right;
    const innerHeight = height - margin.top - margin.bottom;

    const xScale = scaleLinear()
        .domain(extent(dots, dots[0]))
        .range([0, innerWidth])
        .nice();

    const yScale = scaleLinear();
    yScale.domain(extent(dots, dots[1]));
    yScale.range([innerHeight, 0]);
    yScale.nice();

    const g = selection.selectAll('.container').data([null]);
    const gEnter = g
        .enter().append('g')
        .attr('class', 'container');
    gEnter
        .merge(g)
        .attr('transform',
            `translate(${margin.left},${margin.top})`
        );

    //设置x,y轴
    const xAxis = axisBottom(xScale)
        .tickSize(-innerHeight)
        .tickPadding(15);

    const yAxis = axisLeft(yScale)
        .tickSize(-innerWidth)
        .tickPadding(10);

    const yAxisG = g.select('.y-axis');
    const yAxisGEnter = gEnter
        .append('g')
        .attr('class', 'y-axis');
    yAxisG
        .merge(yAxisGEnter)
        .call(yAxis)
        .selectAll('.domain').remove();

    const yAxisLabelText = yAxisGEnter
        .append('text')
        .attr('class', 'axis-label')
        .attr('y', -93)
        .attr('fill', 'black')
        .attr('transform', `rotate(-90)`)
        .attr('text-anchor', 'middle')
        .merge(yAxisG.select('.axis-label'))
        .attr('x', -innerHeight / 2)
        .text(yAxisLabel);


    const xAxisG = g.select('.x-axis');
    const xAxisGEnter = gEnter
        .append('g')
        .attr('class', 'x-axis');
    xAxisG
        .merge(xAxisGEnter)
        .attr('transform', `translate(0,${innerHeight})`)
        .call(xAxis)
        .selectAll('.domain').remove();

    const xAxisLabelText = xAxisGEnter
        .append('text')
        .attr('class', 'axis-label')
        .attr('y', 75)
        .attr('fill', 'black')
        .merge(xAxisG.select('.axis-label'))
        .attr('x', innerWidth / 2)
        .text(xAxisLabel);



};

export const restart=()=>{
     flag = false;
     d3.select("#restart").attr("disabled", "disabled");

     groups.forEach(function(g) {
         g.dots = [];
         g.center.x = g.init.center.x;
         g.center.y = g.init.center.y;
     });

    initdots();

}

export const initdots=(circleRadius)=>{
    for( let i=0;i<K;i++) {
        var x,y;
        var centerX=Math.random()*100;
        var centerY=Math.random()*100;
        var batch=N/K;
        for(let j=0;j<batch;j++){
            x=Math.random()*50+centerX;
            y=Math.random()*50+centerY;
            var dot={
                x: Math.random()*50+centerX,
                y: Math.random()*50+centerY,
                group:undefined,
            }
            dots.push(dot);
        }
    };
    const circles = d3.selectAll('circle').data(dots);
    circles
        .enter().append('circle')
        .attr('cx', innerWidth / 2)//这里让球先在中间点
        .attr('cy', innerHeight / 2)
        .attr('r', 0)
        .merge(circles)
        .transition().duration(3000)//移动花的时间
        .delay((d, i) => i%100* 10)
        .attr('cy', d => d.y)
        .attr('cx', d => d.x)
        .attr('r', circleRadius);
}

export const step=()=>{
    d3.select("#restart").attr("disabled", null);
    if (flag) {
        moveCenter();//把group里面的center的位置都变成当前计算出的位置
        draw();
    } else {
        updateGroups();
        draw();
    }
    flag=!flag;//每次先画线，下一次移动中心点
}



export const init=()=>{
    d3.select("#restart").attr("disabled", "disabled");//restart 按钮设置为不能用
    var N = parseInt(d3.select('#nValue').property('value'), 10);
    var K = parseInt(d3.select('#kValue').property('value'), 10);

    for(var i =0;i<K;i++){
        var g={
            dots:[],
            color: 'hsl(' + (i * 360 / K) + ',100%,50%)',
            center:{
                x: Math.random()*svg.attr('width'),
                y: Math.random()*svg.attr('height')
            },
            init:{
                center:{}
            }
        };
        groups.push(g);
    }
    dots = [];
    flag = false;
    initdots();

}


//draw是先更新了连接的路径和颜色，然后检查了是否有新的组，有就加上标签
export const draw=()=>{
    var circles = dotg.selectAll('circle')
        .data(dots);
    circles.enter()
        .append('circle');
    circles.exit().remove();
    circles
        .transition()
        .duration(500)
        .attr('fill', function(d) { return d.group ? d.group.color : '#ffffff'; })

//如果有dot有分组就画线
    if (dots[0].group) {
        var l = lineg.selectAll('line')
            .data(dots);
        var updateLine = function(lines) {
            lines
                .attr('x1', function(d) { return d.x; })
                .attr('y1', function(d) { return d.y; })
                .attr('x2', function(d) { return d.group.center.x; })
                .attr('y2', function(d) { return d.group.center.y; })
                .attr('stroke', function(d) { return d.group.color; });
        };
        //对着数据画线，enter就是数据部分，还没变成元素的数据部分
        updateLine(l.enter().append('line'));
        updateLine(l.transition().duration(500));
        l.exit().remove();
    } else {
        lineg.selectAll('line').remove();
    }
//path 就是中心点的图案
    var c = centerg.selectAll('path')
        .data(groups);
    var updateCenters = function(centers) {
        centers
            .attr('transform', function(d) { return "translate(" + d.center.x + "," + d.center.y + ") rotate(45)";})//旋转四十五度是形成X的图案
            .attr('fill', function(d,i) { return d.color; })
            .attr('stroke', '#aabbcc');//是X的边框
    };
    c.exit().remove();
    updateCenters(c.enter()
        .append('path')
        .attr('d', d3.svg.symbolCross)
        .attr('stroke', '#aabbcc'));
    updateCenters(c
        .transition()
        .duration(500));//这样传入的还是c而不是函数，函数是设置一下
}


//更新中心点与组的颜色相同
export const updateCenters=(centers)=>{//这里面函数的d没看懂，d是哪来的？d就是centers里面的数据
    centers
        .attr('transform',(d)=>{ return "translate(" + d.center.x + "," + d.center.y + ") rotate(45)";})
        .attr('fill', function(d,i) { return d.color; })
        .attr('stroke', '#aabbcc');
}


//dots里面的值累加
const moveCenter=()=>{
    groups.forEach(function(group, i) {
        if (group.dots.length == 0) return;

        // get center of gravity
        var x = 0, y = 0;
        group.dots.forEach(function(dot) {
            x += dot.x;
            y += dot.y;
        });

        group.center = {
            x: x / group.dots.length,
            y: y / group.dots.length
        };
    });
}




const updateGroups=()=>{
    groups.forEac